@@ -1,9 +1,9 @@
 from . import datatools
 from . import fmn_plottools
-from . import fmn_datatools_v2
+from . import fmn_datatools
 
 __all__ = [
     'datatools',
     'fmn_plottools',
-    'fmn_datatools_v2'
+    'fmn_datatools.py'
 ]
