@@ -7,6 +7,7 @@ import numpy as np
 import plotly.graph_objects as go
 import os
 from PIL import Image
+from datetime import date, datetime
 
 from jupyter_dash import JupyterDash
 import dash_core_components as dcc
@@ -15,6 +16,42 @@ from dash.dependencies import Input, Output, State
 from dash import callback_context
 
 
+def _save_path(data: str, qubit_name: str = 'qubit_default_name',
+               default_path: str = 'D:/Scripts/Measurement_automation/data/qubits/'):
+
+    current_date = str(date.today())
+
+    parent_dir = default_path
+    dir_qubit = os.path.join(default_path, str(qubit_name))
+    dir_date = os.path.join(dir_qubit, current_date)
+    dir_tts = os.path.join(dir_date, 'two_tone_spectroscopy')
+
+    if os.path.exists(parent_dir):
+
+        "checking qubit dir existance"
+        if not os.path.exists(dir_qubit):
+            os.mkdir(dir_qubit)
+        else:
+            pass
+
+        "checking date dir existance"
+        if not os.path.exists(dir_date):
+            os.mkdir(dir_date)
+        else:
+            pass
+
+        "checking tts dir existance"
+        if not os.path.exists(dir_tts):
+            os.mkdir(dir_tts)
+        else:
+            pass
+
+        dir_final = os.path.join(dir_tts, data)
+        return dir_final
+    else:
+        return data
+
+
 class Heatmap:
     """
     Heatmap class provides easy-to-use workspace for building interactive two tone spectroscopy plots
@@ -28,14 +65,22 @@ class Heatmap:
                     cmap: Union[str, Iterable] = ['#ffd200', '#cb2d3e'],
                     title: str = 'Two Tone Spectroscopy',
                     sub_title: str = None,
-                    x_axis_title: str = 'Currents, A',
-                    y_axis_title: str = 'Frequencies, Hz',
+                    x_axis_title: str = 'Voltages, V',
+                    x_axis_size: int = 13,
+                    y_axis_title: str = 'Frequencies, GHz',
+                    y_axis_size: int = 13,
                     colorbar_text: str = None,
                     logo: bool = False,
-                    logo_local = False,
+                    logo_local=False,
+                    y_logo_pos: float = 1.09,
                     save_png: str = None,
-                    save_svg: str = None
+                    save_svg: str = None,
+                    qubit_number: Union[str, float] = None,
+                    exponent_mode = False,
+                    width = 700,
+                    height = 600
                     ):
+
         """
         Built a complete plot of the Two Tone Spectroscopy
         :param data: (optional) Pandas data frame with 3 columns (x, y, z)
@@ -91,7 +136,7 @@ class Heatmap:
 
         """Adding subtitle"""
         Sub_title = '' if sub_title is None else '<br>' + '<i>' + sub_title + '</i>'
-        Title = title + Sub_title
+        Title = title + Sub_title if qubit_number is None else title + ' (qubit ' + str(qubit_number) + ')' + Sub_title
 
         """Choosing colors"""
         if theme.lower() == 'white':
@@ -128,14 +173,33 @@ class Heatmap:
             font=dict(family='Open Sans', color=title_clr),
             xaxis_title=x_axis_title,
             yaxis_title=y_axis_title,
-            autosize=True,
+            autosize=False,
             separators='.',
             paper_bgcolor=paper_clr,
             plot_bgcolor=bg_clr)
 
+        fig.update_yaxes(title_font={"size": y_axis_size}, tickfont_size=y_axis_size)
+        fig.update_xaxes(title_font={"size": x_axis_size}, tickfont_size=x_axis_size)
+
+        """figure size"""
+        # if width is not None:
+        #     fig.update_layout(width=width)
+        # elif height is not None:
+        #     fig.update_layout(height=height)
+        # elif (width is not None) and (height is not None):
+        #     fig.update_layout(width=width, height=height)
+        # else:
+        #     pass
+        fig.update_layout(width=width, height=height)
+
         """upd numbers formatting. See more https://plotly.com/python/tick-formatting/"""
-        fig.update_layout(yaxis_tickformat='.2e', xaxis_tickformat='.2e')
-        fig.update_traces(zhoverformat='.2f')
+        if not exponent_mode:
+            fig.update_layout(yaxis=dict(showexponent='none', exponentformat='e'))
+            fig.update_traces(zhoverformat='.2f')
+            #fig.update_layout(xaxis=dict(showexponent='none', exponentformat='e'))
+        else:
+            fig.update_layout(yaxis_tickformat='.2e', xaxis_tickformat='.2e')
+            fig.update_traces(zhoverformat='.2f')
 
         """add color bar text"""
         if colorbar_text is not None:
@@ -149,7 +213,7 @@ class Heatmap:
                 dict(
                     source='https://raw.githubusercontent.com/ikaryss/pyquac/Master/images/logo_sign.png',
                     xref="paper", yref="paper",
-                    x=1, y=1.09,
+                    x=1, y=y_logo_pos,
                     sizex=0.09,
                     sizey=0.09,
                     xanchor="right", yanchor="bottom",
@@ -164,7 +228,7 @@ class Heatmap:
                 dict(
                     source=img,
                     xref="paper", yref="paper",
-                    x=1, y=1.09,
+                    x=1, y=y_logo_pos,
                     sizex=0.09,
                     sizey=0.09,
                     xanchor="right", yanchor="bottom",
@@ -173,7 +237,6 @@ class Heatmap:
         else:
             pass
 
-
         """grid color. See more https://plotly.com/python/axes/"""
         fig.update_layout(xaxis=dict(showgrid=True, gridcolor=grid_clr, gridwidth=0.1,
                                      zeroline=True, zerolinewidth=0.1, zerolinecolor=grid_clr),
@@ -194,12 +257,18 @@ class Heatmap:
                         cmap: Union[str, Iterable] = ['#ffd200', '#cb2d3e'],
                         title: str = 'Two Tone Spectroscopy',
                         sub_title: str = None,
-                        x_axis_title: str = 'Currents, A',
-                        y_axis_title: str = 'Frequencies, Hz',
+                        x_axis_title: str = 'Voltages, V',
+                        x_axis_size: int = 13,
+                        y_axis_title: str = 'Frequencies, GHz',
+                        y_axis_size: int = 13,
                         colorbar_text: str = None,
                         logo: bool = False,
                         logo_local=False,
-                        y_logo_pos: float = 1.09
+                        y_logo_pos: float = 1.09,
+                        qubit_number: Union[str, float] = None,
+                        exponent_mode=False,
+                        width=None,
+                        height=None
                         ):
         """
         Built a complete plot formatter for your go.Fig object.
@@ -242,7 +311,7 @@ class Heatmap:
 
         """Adding subtitle"""
         Sub_title = '' if sub_title is None else '<br>' + '<i>' + sub_title + '</i>'
-        Title = title + Sub_title
+        Title = title + Sub_title if qubit_number is None else title + ' (qubit ' + str(qubit_number) + ')' + Sub_title
 
         """Choosing colors"""
         if theme.lower() == 'white':
@@ -282,14 +351,32 @@ class Heatmap:
             font=dict(family='Open Sans', color=title_clr),
             xaxis_title=x_axis_title,
             yaxis_title=y_axis_title,
-            autosize=True,
+            autosize=False,
             separators='.',
             paper_bgcolor=paper_clr,
             plot_bgcolor=bg_clr)
 
+        fig.update_yaxes(title_font={"size": y_axis_size}, tickfont_size=y_axis_size)
+        fig.update_xaxes(title_font={"size": x_axis_size}, tickfont_size=x_axis_size)
+
+        """figure size"""
+        if width is not None:
+            fig.update_layout(width=width)
+        elif height is not None:
+            fig.update_layout(height=height)
+        elif (width is not None) and (height is not None):
+            fig.update_layout(width=width, height=height)
+        else:
+            pass
+
         """upd numbers formatting. See more https://plotly.com/python/tick-formatting/"""
-        fig.update_layout(yaxis_tickformat='.2e', xaxis_tickformat='.2e')
-        fig.update_traces(zhoverformat='.2f')
+        if not exponent_mode:
+            fig.update_layout(yaxis=dict(showexponent='none', exponentformat='e'))
+            # fig.update_layout(xaxis=dict(showexponent='none', exponentformat='e'))
+            fig.update_traces(zhoverformat='.2f')
+        else:
+            fig.update_layout(yaxis_tickformat='.2e', xaxis_tickformat='.2e')
+            fig.update_traces(zhoverformat='.2f')
 
         """add color bar text"""
         if colorbar_text is not None:
@@ -318,7 +405,7 @@ class Heatmap:
                 dict(
                     source=img,
                     xref="paper", yref="paper",
-                    x=1, y=1.09,
+                    x=1, y=y_logo_pos,
                     sizex=0.09,
                     sizey=0.09,
                     xanchor="right", yanchor="bottom",
@@ -335,15 +422,16 @@ class Heatmap:
 
         pass
 
-
     """
     info https://plotly.com/python/static-image-export/
     requirements:
     conda install -c conda-forge python-kaleido
     
     """
+
     @staticmethod
-    def save_fig_as(fig, name: str = 'figure.svg', method: str = 'image', create_folder: bool = False, folder_name: str = 'images'):
+    def save_fig_as(fig, name: str = 'figure.svg', method: str = 'image', create_folder: bool = False,
+                    folder_name: str = 'images'):
         """
 
         :param fig:
@@ -358,7 +446,7 @@ class Heatmap:
             if not os.path.exists(folder_name):
                 os.mkdir(folder_name)
 
-            directory = folder_name+'/'+name
+            directory = folder_name + '/' + name
 
         else:
             directory = name
@@ -373,15 +461,15 @@ class Heatmap:
 class Dash_app:
 
     @classmethod
-    def configure_app(cls, data, fig, mode='inline', port=8051, interval=5e3):
+    def configure_app(cls, data, fig, mode='inline', port=8051, interval=5e3, qubit_number=None):
         cls.fig = fig
         cls.mode = mode
         cls.interval = interval
         cls.port = port
+        cls.qubit_number = qubit_number
 
         disabled_btn = False
         disabled_input = False
-        #interval_interval = 5e3
         maxx_interval = -1
 
         # Build App
@@ -397,11 +485,11 @@ class Dash_app:
                               value=['YES', 'NO'],
                               labelStyle={'display': 'inline-block'}
                               ),
-                html.Button('Refresh', id='btn-nclicks-4', n_clicks=0),
+                html.Button('Manual upd', id='btn-nclicks-4', n_clicks=0),
                 dcc.Input(
                     id="input_time",
                     type='number',
-                    placeholder="Interval, ms",
+                    placeholder="Auto upd in, ms",
                     min=800,
                 ),
                 dcc.Graph(id="heatmap", figure=cls.fig),
@@ -438,13 +526,33 @@ class Dash_app:
 
             changed_id = [p['prop_id'] for p in callback_context.triggered][0]
             if 'btn-nclicks-1' in changed_id:
-                data.get_result().to_csv('present_data.csv', index=False)
+
+                file_name = ('TTS_' + 'q' + str(cls.qubit_number) + '_0' + str(datetime.now().strftime(
+                    "_%H-%M-%S")) + '.csv' if cls.qubit_number is not None else 'TTS_' + 'q' + '_0' + str(
+                    datetime.now().strftime("_%H-%M-%S")) + '.csv')
+
+                data.get_result().to_csv(_save_path(file_name, cls.qubit_number), index=False)
             elif 'btn-nclicks-2' in changed_id:
-                fig.write_image('fig1.pdf')
+
+                file_name = ('TTS_'+'q'+str(cls.qubit_number)+'_0'+str(datetime.now().strftime(
+                    "_%H-%M-%S"))+'.pdf' if cls.qubit_number is not None else 'TTS_'+'q'+'_0'+str(
+                    datetime.now().strftime("_%H-%M-%S"))+'.pdf')
+
+                fig.write_image(_save_path(file_name, cls.qubit_number))
             elif 'btn-nclicks-3' in changed_id:
-                fig.write_html('fig1.html')
+
+                file_name = ('TTS_' + 'q' + str(cls.qubit_number) + '_0' + str(datetime.now().strftime(
+                    "_%H-%M-%S")) + '.html' if cls.qubit_number is not None else 'TTS_' + 'q' + '_0' + str(
+                    datetime.now().strftime("_%H-%M-%S")) + '.html')
+
+                fig.write_html(_save_path(file_name, cls.qubit_number))
             elif 'btn-nclicks-5' in changed_id:
-                fig.write_image('fig1.svg')
+
+                file_name = ('TTS_' + 'q' + str(cls.qubit_number) + '_0' + str(datetime.now().strftime(
+                    "_%H-%M-%S")) + '.svg' if cls.qubit_number is not None else 'TTS_' + 'q' + '_0' + str(
+                    datetime.now().strftime("_%H-%M-%S")) + '.svg')
+
+                fig.write_image(_save_path(file_name, cls.qubit_number))
 
             elif 'btn-nclicks-4' in changed_id:
                 z = data.njit_result
